# AnalysisD.tcl - takes output from dynamic analyses
# Units: kip, in
# Kevin Mackie, 2001/06/06
# mackie@ce.berkeley.edu
#
# Takes output from G3 dynamic analyses (called from bridge
# input file) and outputs data to files for computation of UCSD
# output quantities

proc AnalysisD {skew L LoH fy fc rho_s DcDs ksoil wt rhos_trans eqknumin param_num ida ida_cnt} {

# these constants are from GenerateBridge.tcl
set cover 1.5
set trans_bar 6.0
set deck_depth 7.0

# generate appropriate vaiables from input parameters
set pi [expr acos(-1.0)]
set Ds [expr $deck_depth*12.0]
set Dcol [expr $Ds*$DcDs]
set Ht [expr $L/$LoH]

# ------------------------------------------------
# filenames of dynamic nodal displacements for drift data
if { $ida == 1 } {
   set fstub [format "d%04i%04i%02i" $param_num $eqknumin $ida_cnt]
} else {
   set fstub [format "d%04i%04i" $param_num $eqknumin]
}
set delf [format "%snd08.out" $fstub]

source MaxVal.tcl
set dmax [MaxVal $delf 4 1]
scan $dmax "%f %f %f %f %f %f %f %f" temp max_x max_y max_z temp resid_x resid_y resid_z

# ------------------------------------------------
# filenames of dynamic element info for curvature data
set curd [format "%ssd015.out" $fstub]
set curf [format "%ssf015.out" $fstub]

set kmax [MaxVal $curd 4 0]
set mmax [MaxVal $curf 4 0]
scan $kmax "%f %f %f %f" max_ex max_kx max_ky max_kz
scan $mmax "%f %f %f %f" max_Nx max_Mx max_My max_Mz
# note: for curvature information, the term
# x actually refers to the transverse direction
# y refers to the longitudinal direction

# ------------------------------------------------
# load fiber information and determine max steel and concrete strain
set mat_file [open "fiber_numbers" "r"]
gets $mat_file line
scan $line "%i %i" conc_fibers steel_fibers
close $mat_file

set maxste 0.0
set maxcne 0.0
set maxsts 0.0
set maxcns 0.0
for {set i 20} {$i < $conc_fibers} {incr i} {
   set strain_file [open [format "%sfdc%03i.out" $fstub $i] "r"]
   set stress_file [open [format "%sffc%03i.out" $fstub $i] "r"]
   while {-1 != [gets $strain_file lined] } {
      gets $stress_file linef
      scan $lined  "%f" temp_eps
      scan $linef  "%f" temp_sig
      if { [expr abs($temp_eps)] > $maxcne} {
         if { [expr abs($temp_sig)] > 1 } {
            set maxcne [expr abs($temp_eps)]
         }
      }
      if { [expr abs($temp_sig)] > $maxcns} {
         set maxcns [expr abs($temp_sig)]
      }
   }
   close $strain_file
   close $stress_file

   file delete [format "%sfdc%03i.out" $fstub $i]
   file delete [format "%sffc%03i.out" $fstub $i]
}

for {set i $conc_fibers} {$i < $conc_fibers+$steel_fibers} {incr i} {
   set strain_file [open [format "%sfds%03i.out" $fstub $i] "r"]
   set stress_file [open [format "%sffs%03i.out" $fstub $i] "r"]
   while {-1 != [gets $strain_file lined] } {
      gets $stress_file linef
      scan $lined  "%f" temp_eps
      scan $linef  "%f" temp_sig
      if { [expr abs($temp_eps)] > $maxste} {
         if { [expr abs($temp_sig)] > 1 } {
            set maxste [expr abs($temp_eps)]
         }
      }
      if { [expr abs($temp_sig)] > $maxsts} {
         set maxsts [expr abs($temp_sig)]
      }
   }
   close $strain_file
   close $stress_file

   file delete [format "%sfds%03i.out" $fstub $i]
   file delete [format "%sffs%03i.out" $fstub $i]
}

file delete "fiber_numbers"

# ------------------------------------------------
# load moment rotation for numerical integration of dissipated energy
# note: the maximum rotation does not occur at the node at the
# top of the column, ie, plastic hinging is occurring, so theoretically,
# we want M-Theta somewhere near the top of the column, so just use
# assumption that plastic curvature is acting over plastic hinge length
# of h/2 (research more accurate values for circular columns)
# CalTrans: Lp=0.08L+0.15fy*dbl >= 0.3fy*dbl
#  where L=length from max M to inflection point
#        dbl=nominal longitudinal rebar diameter
set lp [expr 0.3*$fy*11/8.0+1.0]

set curd1 [format "%ssd015.out" $fstub]
set curf1 [format "%ssf015.out" $fstub]

source Energy.tcl
set e1 [Energy $curf1 $curd1 $lp]
scan $e1 "%f %f %f" E_totalx E_totaly E_totalz

# ------------------------------------------------
# prepare data as necessary

# Drift ratios
set drift_x [expr abs($max_x/$Ht*100)]
set drift_y [expr abs($max_y/$Ht*100)]

# ------------------------------------------------
# write table of summary information to file
if { $ida == 1 } {
   set fstub [format "fd%04i%04i%02i" $param_num $eqknumin $ida_cnt]
} else {
   set fstub [format "fd%04i%04i" $param_num $eqknumin]
}
set file [open [format "%s.out" $fstub] "w"]

puts $file "Max_displ. in x        [expr abs($max_x)]"
puts $file "Max_displ. in y        [expr abs($max_y)]"
puts $file "Max_displ. in z        [expr abs($max_z)]"
puts $file "Drift in x             $drift_x"
puts $file "Drift in y             $drift_y"
puts $file "Max_curv in long.      [expr $max_ky]"
puts $file "Max_curv in trans.     [expr $max_kx]"
puts $file "Max_moment in long.    [expr $max_My]"
puts $file "Max_moment in trans.   [expr $max_Mx]"
puts $file "Max steel strain       [expr $maxste]"
puts $file "Max conc strain        [expr $maxcne]"
puts $file "Max steel stress       [expr $maxsts]"
puts $file "Max conc stress        [expr $maxcns]"
puts $file "residual in x          [expr abs($resid_x)]"
puts $file "residual in y          [expr abs($resid_y)]"
puts $file "residual in z          [expr abs($resid_z)]"
puts $file "Hysteretic energy ln.  $E_totaly"
puts $file "Hysteretic energy tr.  $E_totalx"
puts $file "Hysteretic energy vt.  $E_totalz"

close $file

}
